@@ -624,15 +624,15 @@ build_time_vars = {'ABIFLAGS': '',
                    '/usr/include/python3.12',
  'INCLUDEDIR': '/usr/include',
  'INCLUDEPY': '/usr/include/python3.12',
- 'INSTALL': '/var/tmp/portage/._portage_reinstall_.2z6ykai_/bin/ebuild-helpers/xattr/install '
+ 'INSTALL': '/var/tmp/portage/._portage_reinstall_.xdzukwzt/bin/ebuild-helpers/xattr/install '
             '-c',
- 'INSTALL_DATA': '/var/tmp/portage/._portage_reinstall_.2z6ykai_/bin/ebuild-helpers/xattr/install '
+ 'INSTALL_DATA': '/var/tmp/portage/._portage_reinstall_.xdzukwzt/bin/ebuild-helpers/xattr/install '
                  '-c -m 644',
- 'INSTALL_PROGRAM': '/var/tmp/portage/._portage_reinstall_.2z6ykai_/bin/ebuild-helpers/xattr/install '
+ 'INSTALL_PROGRAM': '/var/tmp/portage/._portage_reinstall_.xdzukwzt/bin/ebuild-helpers/xattr/install '
                     '-c',
- 'INSTALL_SCRIPT': '/var/tmp/portage/._portage_reinstall_.2z6ykai_/bin/ebuild-helpers/xattr/install '
+ 'INSTALL_SCRIPT': '/var/tmp/portage/._portage_reinstall_.xdzukwzt/bin/ebuild-helpers/xattr/install '
                    '-c',
- 'INSTALL_SHARED': '/var/tmp/portage/._portage_reinstall_.2z6ykai_/bin/ebuild-helpers/xattr/install '
+ 'INSTALL_SHARED': '/var/tmp/portage/._portage_reinstall_.xdzukwzt/bin/ebuild-helpers/xattr/install '
                    '-c -m 755',
  'INSTSONAME': 'libpython3.12.so.1.0',
  'IO_H': 'Modules/_io/_iomodule.h',
@@ -790,7 +790,7 @@ build_time_vars = {'ABIFLAGS': '',
  'MODULE_PYEXPAT_DEPS': '',
  'MODULE_PYEXPAT_LDFLAGS': '-lexpat',
  'MODULE_PYEXPAT_STATE': 'yes',
- 'MODULE_READLINE_CFLAGS': '-D_POSIX_C_SOURCE=199506L',
+ 'MODULE_READLINE_CFLAGS': '',
  'MODULE_READLINE_LDFLAGS': '-lreadline',
  'MODULE_READLINE_STATE': 'yes',
  'MODULE_RESOURCE_STATE': 'yes',
@@ -876,13 +876,10 @@ build_time_vars = {'ABIFLAGS': '',
  'MODULE__CTYPES_STATE': 'yes',
  'MODULE__CTYPES_TEST_LDFLAGS': '-lm',
  'MODULE__CTYPES_TEST_STATE': 'yes',
- 'MODULE__CURSES_CFLAGS': '-D_POSIX_C_SOURCE=199506L -DNCURSES_WIDECHAR '
-                          '-I/usr/include/ncursesw',
+ 'MODULE__CURSES_CFLAGS': '-DNCURSES_WIDECHAR -I/usr/include/ncursesw',
  'MODULE__CURSES_LDFLAGS': '-lncursesw -ltinfow',
- 'MODULE__CURSES_PANEL_CFLAGS': '-I/usr/include/ncursesw '
-                                '-D_POSIX_C_SOURCE=199506L -DNCURSES_WIDECHAR '
-                                '-D_POSIX_C_SOURCE=199506L -DNCURSES_WIDECHAR '
-                                '-I/usr/include/ncursesw',
+ 'MODULE__CURSES_PANEL_CFLAGS': '-I/usr/include/ncursesw -DNCURSES_WIDECHAR '
+                                '-DNCURSES_WIDECHAR -I/usr/include/ncursesw',
  'MODULE__CURSES_PANEL_LDFLAGS': '-lpanelw -lncursesw -ltinfow',
  'MODULE__CURSES_PANEL_STATE': 'yes',
  'MODULE__CURSES_STATE': 'yes',
